# frozen_string_literal: true

module Bulkrax::HasLocalProcessing
  # This method is called during build_metadata
  # add any special processing here, for example to reset a metadata property
  # to add a custom property from outside of the import data
  def add_local
    if record['model'] == 'Collection'
      parsed_metadata['collection_type_gid'] = "gid://hyrax3-app/Hyrax::CollectionType/#{collection_type.id}"
    end

    return unless record['model'] == 'CrcDataset'

    map_multiple_subject
    map_multiple_modality

    parsed_metadata['complex_person'] = complex_person
    parsed_metadata['complex_identifier'] = complex_identifier
    parsed_metadata['complex_date'] = complex_date
    parsed_metadata['complex_subject'] = [complex_subject]
    parsed_metadata['complex_funding_reference'] = [complex_funding_reference]
  end

  def map_multiple_subject
    parsed_metadata['subject'] = record['subject'].present? ? record['subject'].split(';') : []
  end

  def map_multiple_modality
    parsed_metadata['modality'] = record['modality'].present? ? JSON.parse(record['modality']) : []
  end

  def collection_type
    Hyrax::CollectionType.find_by_title(record['collection_type'])
  end

  def complex_person
    complex_persons = []

    %w[creator contributor].each do |role|
      next unless record[role.to_s].present?

      full_names = record[role.to_s].split(';')
      orcids = record.fetch("#{role}_orcid", '').split(';')
      affiliations = record.fetch("#{role}_affilation", '').split(';')
      # ignoring creator identifier scheme and contributor identifier scheme
      _schemes = record.fetch("#{role}_scheme", '').split(';')

      full_names.each_with_index do |full_name, index|
        complex_person_object = ComplexPerson.new(RDF::Node.new, ActiveTriples::Resource.new)

        complex_person_object['last_name'], complex_person_object['first_name'] = full_name.split(',').map(&:strip)
        complex_person_object['name'] = full_name
        complex_person_object['role'] = role
        complex_person_object['orcid'] = orcids[index] if orcids.count == full_names.count
        complex_person_object['affiliation'] = affiliations[index] if affiliations.count == full_names.count

        complex_persons << complex_person_object
      end
    end

    complex_persons
  end

  def complex_identifier
    complex_identifiers = []

    return complex_identifiers unless record['group_identifier'].present?

    record['group_identifier'].split(';').each do |id|
      complex_identifier_object = ComplexIdentifier.new(RDF::Node.new, ActiveTriples::Resource.new)

      complex_identifier_object['scheme'] = 'group id'
      complex_identifier_object['identifier'] = id

      complex_identifiers << complex_identifier_object
    end

    complex_identifiers
  end

  def complex_date
    complex_dates = []

    # record_date
    record_dates = record.fetch('record_date', '').split(';')

    record_dates.each do |record_date|
      complex_date_object = ComplexDate.new(RDF::Node.new, ActiveTriples::Resource.new)

      complex_date_object['description'] = 'Recorded'
      complex_date_object['date'] = record_date

      complex_dates << complex_date_object
    end

    # Datacite date and date type
    date_types = record.fetch('date_description', '').split(';')
    dates =  record.fetch('date', '').split(';')
    return complex_dates unless date_types.count == dates.count

    date_types.each_with_index do |date_type, index|
      typ = DateService.new.find_by_label(date_type)
      next unless typ.present?

      complex_date_object = ComplexDate.new(RDF::Node.new, ActiveTriples::Resource.new)

      complex_date_object['description'] = typ['id']
      complex_date_object['date'] = dates[index]

      complex_dates << complex_date_object
    end

    complex_dates
  end

  def complex_subject
    complex_subject_object = ComplexSubject.new(RDF::Node.new, ActiveTriples::Resource.new)

    %w[subject_identifier subject_species subject_type subject_sex subject_age].each do |attribute|
      complex_subject_object[attribute] = record[attribute]
    end

    complex_subject_object
  end

  def complex_funding_reference
    complex_funding_reference_object = ComplexFundingReference.new(RDF::Node.new, ActiveTriples::Resource.new)

    %w[funder_identifier funder_name award_number award_uri award_title].each do |attribute|
      complex_funding_reference_object[attribute] = record[attribute]
    end

    complex_funding_reference_object
  end

end
