# frozen_string_literal: true

require 'rails_helper'
require 'json'

RSpec.describe CrcDatasetIndexer do
  describe 'indexes a date active triple resource with all the attributes' do
    before do
      dates = [
        {
          date: '1988-10-28',
          description: 'http://bibframe.org/vocab/providerDate',
        }, {
          date: '2018-01-01'
        }, {
          description: 'http://bibframe.org/vocab/changeDate'
        }, {
          description: 'http://purl.org/dc/terms/dateAccepted',
          date: ''
        }
      ]

      obj = build(:crc_dataset, complex_date_attributes: dates)

      @solr_document = obj.to_solr
    end

    it 'rejects blank dates' do
      expect(@solr_document).not_to include('complex_date_updated_ssm')
      expect(@solr_document).not_to include('complex_year_updated_sim')
      expect(@solr_document).not_to include('complex_date_accepted_ssm')
      expect(@solr_document).not_to include('complex_year_accepted_sim')
    end

    it 'indexes as displayable' do
      expect(@solr_document).to include('complex_date_ssm')
      expect(JSON.parse(@solr_document['complex_date_ssm'])).not_to be_empty
    end

    it 'indexes as dateable' do
      expect(@solr_document['complex_date_dtsim']).to match_array(
        ["1988-10-28T00:00:00Z", "2018-01-01T00:00:00Z"])
    end

    it 'indexes year as facetable' do
      expect(@solr_document['complex_year_sim']).to match_array(
        ["1988", "2018"])
    end

    it 'indexes each type as dateable' do
      expect(@solr_document['complex_date_submitted_dtsim']).to match_array(["1988-10-28T00:00:00Z"])
    end

    it 'indexes each type as displayable' do
      expect(@solr_document['complex_date_submitted_ssm']).to match_array(["1988-10-28"])
    end

    it 'indexes each year as facetable' do
      expect(@solr_document['complex_year_submitted_sim']).to match_array(["1988"])
    end
  end

  describe 'indexes an identifier active triple resource with all the attributes' do
    before do
      ids = [
        {
          identifier: '0000-0000-0000-0000',
          scheme: 'ORCID'
        }, {
          identifier: '1234',
          scheme: 'identifier local'
        }, {
          identifier: '12345345234',
          scheme: 'Orcid'
        }
      ]
      obj = build(:crc_dataset, complex_identifier_attributes: ids)
      
      @solr_document = obj.to_solr
    end

    it 'indexes as displayable' do
      expect(@solr_document).to include('complex_identifier_ssm')
      expect(JSON.parse(@solr_document['complex_identifier_ssm'])).not_to be_empty
    end

    it 'indexes as symbol' do
      expect(@solr_document['complex_identifier_ssim']).to match_array(['0000-0000-0000-0000', '1234', '12345345234'])
    end

    it 'indexes each type as symbol' do
      expect(@solr_document['complex_identifier_orcid_ssim']).to match_array(['0000-0000-0000-0000', '12345345234'])
      expect(@solr_document['complex_identifier_identifier_local_ssim']).to match_array(['1234'])
    end
  end

  describe 'indexes the person active triple resource with all the attributes' do
    before do
      people = [
        {
          first_name: ['Foo'],
          last_name: 'Bar',
          role: "author",
          affiliation: "org",
          orcid: "32123"
        }, {
          name: 'Big Baz',
          role: "editor",
          affiliation: "org 2",
          orcid: "32125"
      }, {
          name: 'Joe Blogg',
          role: "author",
        }, {
          first_name: ['James'],
          last_name: 'Bond',
          name: 'James Bond',
          role: "data depositor",
          affiliation: "org",
          orcid: "32127"
        }
      ]

      obj = build(:crc_dataset, complex_person_attributes: people)

      @solr_document = obj.to_solr
    end

    it 'indexes person as displayable' do
      expect(@solr_document).to include('complex_person_ssm')
      expect(JSON.parse(@solr_document['complex_person_ssm'])).not_to be_empty
    end

    it 'indexes name as facetable' do
      expect(@solr_document['complex_person_sim']).to match_array(['Foo Bar', 'Big Baz', 'Joe Blogg', 'James Bond'])
    end

    it 'indexes name as stored searchable' do
      expect(@solr_document['complex_person_tesim']).to match_array(['Foo Bar', 'Big Baz', 'Joe Blogg', 'James Bond'])
    end

    it 'index name by role as stored searchable' do
      expect(@solr_document['complex_person_author_tesim']).to match_array(['Foo Bar', 'Joe Blogg'])
      expect(@solr_document['complex_person_editor_tesim']).to match_array(['Big Baz'])
      expect(@solr_document['complex_person_data_depositor_tesim']).to match_array(['James Bond'])
    end

    it 'index name by role as facetable' do
      expect(@solr_document['complex_person_author_sim']).to match_array(['Foo Bar', 'Joe Blogg'])
      expect(@solr_document['complex_person_editor_sim']).to match_array(['Big Baz'])
      expect(@solr_document['complex_person_data_depositor_sim']).to match_array(['James Bond'])
    end

    it 'indexes the identifier as a symbol' do
      puts @solr_document.keys()
      expect(@solr_document['complex_person_identifier_ssim']).to match_array(['32123', '32125', '32127'])
    end

    it 'indexes affiliation as stored searchable' do
      expect(@solr_document['complex_person_affiliation_tesim']).to match_array(['org', 'org 2'])
    end

    it 'indexes affiliation as facetable' do
      expect(@solr_document['complex_person_affiliation_sim']).to match_array(['org', 'org 2'])
    end
  end

  describe 'indexes the funding reference active triple resource with all the attributes' do
    before do
      fund_ref = [
        {
          funder_identifier: 'f12345',
          funder_name: 'Bar',
          award_number: 'c',
          award_title: 'Title of the award'
        },
        {
          funder_identifier: 'f22345',
          funder_name: 'Baz',
          award_number: 'a223345',
          award_title: 'Another award'
        }
      ]

      obj = build(:crc_dataset, complex_funding_reference_attributes: fund_ref)

      @solr_document = obj.to_solr
    end

    it 'indexes as displayable' do
      expect(@solr_document).to include('complex_funding_reference_ssm')
      expect(JSON.parse(@solr_document['complex_funding_reference_ssm'])).not_to be_empty
    end

    it 'indexes funder name as stored searchable' do
      expect(@solr_document['funder_tesim']).to match_array(%w[Bar Baz])
    end

    it 'indexes funder name as facetable' do
      expect(@solr_document['funder_sim']).to match_array(%w[Bar Baz])
    end

    it 'indexes award title as stored searchable' do
      expect(@solr_document['award_title_tesim']).to match_array(['Title of the award', 'Another award'])
    end
  end

  describe 'indexes the relation active triple resource with all the attributes' do
    before do
      relationships = [
        {
          title: 'A related item',
          url: 'http://example.com/relation',
          complex_identifier_attributes: [{
            identifier: ['123456'],
            label: ['local']
          }],
          relationship: 'isPartOf'
        }, {
          title: 'A 2nd related item',
          url: 'http://example.com/relation2',
          relationship: 'isPartOf'
        }, {
          title: 'A 3rd relation item',
          url: 'http://example.com/relation3',
          relationship: 'isNewVersionOf'
        }
      ]

      obj = build(:crc_dataset, complex_relation_attributes: relationships)

      @solr_document = obj.to_solr
    end

    it 'indexes as displayable' do
      expect(@solr_document).to include('complex_relation_ssm')
      expect(JSON.parse(@solr_document['complex_relation_ssm'])).not_to be_empty
    end

    it 'indexes the title as stored searchable' do
      expect(@solr_document['complex_relation_title_tesim']).to match_array(
        ['A related item', 'A 2nd related item', 'A 3rd relation item'])
    end

    it 'indexes the relationship as facetable' do
      expect(@solr_document['complex_relation_relationship_sim']).to match_array(
        ['isPartOf', 'isPartOf', 'isNewVersionOf'])
    end

    it 'indexes the relation by relationship as stored searchable' do
      expect(@solr_document['complex_relation_ispartof_tesim']).to match_array(
        ['A related item', 'A 2nd related item'])
      expect(@solr_document['complex_relation_isnewversionof_tesim']).to match_array(
        ['A 3rd relation item'])
    end

    it 'indexes the relation by relationship as facetable' do
      expect(@solr_document['complex_relation_ispartof_sim']).to match_array(
        ['A related item', 'A 2nd related item'])
      expect(@solr_document['complex_relation_isnewversionof_sim']).to match_array(
        ['A 3rd relation item'])
    end
  end
end
